#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 Axigen Messaging SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" Definitions of classes related to CLI Errors
    Error
      InvalidResponseError
      ConnectionClosedError
      InvalidContextError
      DomainNotFoundError
      AccountNotFoundError
"""

__copyright__ = "Copyright (C) 2020, Axigen Messaging SRL"
__license__ = "Apache License Version 2.0"
__version__ = "1.1.0"
__author__ = 'eugen.belea@axigen.com (Eugen Belea)'
__maintainer__ = 'eugen.belea@axigen.com (Eugen Belea)'


try:
    import exceptions
except ImportError:
    import builtins as exceptions

class Error(exceptions.Exception):
    '''
    CLI Error (base class for all CLI errors) 
    '''
    def __init__(self, _msg="Uknown CLI error"):
        self.msg = _msg

    def __str__(self):
        return self.msg 

class InvalidResponseError(Error):
    '''
    Invalid (unexpected) format of a response message
    '''
    def __init__(self, _cmdName=None):
        if _cmdName:
            Error.__init__(self, "InvalidResponse (command '%s')" % _cmdName)
        else:
            Error.__init__(self, "Invalid Response")
            
class ConnectionClosedError(Error):
    def __init__(self, _cmdName=None):
        if _cmdName:
            Error.__init__(self, "ConnectionClosedByServer (command '%s')" % _cmdName)
        else:
            Error.__init__(self, "ConnectionClosedByServer")
    
class InvalidContextError(Error):
    '''
    CLI Invalid Context 
    '''
    def __init__(self):
        Error.__init__(self, "Invalid CLI context found")

class DomainNotFoundError(Error):
    def __init__(self, _domainName):
        Error.__init__(self, "Domain %s not found" % _domainName)


class AccountNotFoundError(Error):
    def __init__(self, _accountName, _domainName=None):
        if _domainName:
            Error.__init__(self, "Account %s@%s not found" % (_accounName, _domainName))
        else:
            Error.__init__(self, "Account %s not found" % _accounName)

        
