#!/usr/bin/env python

from __future__ import print_function

import sys
import os
import string
import random
import optparse
import getpass
import time
import traceback

sys.path.append(os.path.join(os.path.dirname(sys.path[0]), '..'))
from backup import *
from axicli import *

if __name__ == "__main__":

    parser = optparse.OptionParser(description = 'Axigen backup utility.',\
                                   version = 'Axigen backup version 1.1',\
                                   usage = '%prog [options] <mount dir> <backup dir> [<restricted domain>]')
    parser.add_option('-H', '--host', default = 'localhost', help = 'axigen cli service host name (default: <localhost>')
    parser.add_option('-p', '--port', type = 'int', default = 7000, help = 'axigen cli service port (default: 7000)')
    parser.add_option('-u', '--user', default = 'admin', help = 'axigen cli login user name (default: <admin>)')
    parser.add_option('-P', '--password', help = 'axigen cli login password')
    #parser.add_option('mountDir', help = 'FUSE mount directory')
    #parser.add_option('backupDir', help = 'Backup directory')

    (options, args) = parser.parse_args()

    if len(args) < 2:
        parser.print_usage()
        sys.exit(2)

    if options.password == None:
        options.password = getpass.getpass(\
            "Enter password for {0}@{1}".format(options.user, options.host))
    
    srcDir = os.path.realpath(args[0])
    if os.path.exists(srcDir):
        if not os.path.isdir(srcDir):
            print("Source path must be an empty directory")
            sys.exit(1)
        if os.listdir(srcDir):
            print("Source directory not empty")
            sys.exit(1)
    else:
        srcDir = args[0]

    destDir = os.path.realpath(args[1])
    if os.path.exists(destDir) and not os.path.isdir(destDir):
        print("Destination path must be a directory")
        sys.exit(1)
        
    bkbasename = os.path.basename(args[1].rstrip('/\\'))
    if not bkbasename:
        bkbasename = "axibackup-"
    else:
        bkbasename = bkbasename[:9].lower() + "-"

    if len(args) >= 3:
        restrictedDomain = args[2]
    else:
        restrictedDomain = None
        
    bkidxs = set()
    lastbkname = None
    try:
        with open(os.path.join(destDir, "axibackups.txt"), 'r') as bkf:
            for bkn in bkf:
                bkn = bkn.strip().lower()
                if bkn[:len(bkbasename)] == bkbasename:
                    try:
                        bkidx = int(bkn[len(bkbasename)+6:])
                        bkidxs = bkidxs | set([bkidx])
                    except:
                        pass
                if bkn:
                    lastbkname = bkn
    except:                    
        pass
    bkindex = None
    for bkidx in range(10000):
        if not (bkidxs &  set([bkidx])):
            bkindex = bkidx
            break
    if bkindex == None:
        bkname = None
    else:
        rnd=[]
        for c in string.ascii_letters:
            rnd.append(c)
        for c in string.digits:
            rnd.append(c)
        random.shuffle(rnd)

        bkname = "{0}{1}-{2}".format(bkbasename, "".join(rnd[:5]), bkindex)

    try:
        cliConn = cliconnection.Connection(options.user, options.password, options.host, options.port)
        cliConn.login()
        clicommand.ConfigCommand("server").execute(cliConn)
        clicommand.ConfigCommand("ftp-backup").execute(cliConn)
        if lastbkname:
            sinceBk = "since " + lastbkname
        else:
            sinceBk = ""
        if bkname:
            updateBk = "backup " + bkname
        else:
            updateBk = ""
        clicommand.SimpleCommand("filesystem mount {0} {1} {2}".format(srcDir, sinceBk, updateBk)).execute(cliConn)
        cliConn.logout()
        del cliConn
    except:
        print("Error mounting axigen backup filesystem:", sys.exc_info())
        sys.exit(3)

    time.sleep(1)
    bkResult = False
    axiDir = os.path.join(srcDir, "domains")
    if not os.path.isdir(axiDir):
        print("Unable to mount axigen backup filesystem")
    else:
        bkeng = None
        try:
            bkeng = bkengine.Engine(axiDir, destDir)
            bkeng.restrictBackup(restrictDomain = restrictedDomain)
            cpi = bkreporter.getProgressIndicator()
            bkeng.registerProgressIndicator(cpi)
            bkResult = bkeng.backup()
        except:
            print("Unhandled exception during backup:", sys.exc_info()[0])
            traceback.print_tb(sys.exc_info()[2])
            bkResult = False
        del bkeng
        os.closerange(3, 128)

        if bkResult and bkname:
            try:
                with open(os.path.join(destDir, "axibackups.txt"), 'a') as bkf:
                    bkf.write("{0}\n".format(bkname))
            except:
                print("Unable to update backup list file axibackup.txt:", sys.exc_info()[0])

    try:
        cliConn = cliconnection.Connection(options.user, options.password, options.host, options.port)
        cliConn.login()
        clicommand.ConfigCommand("server").execute(cliConn)
        clicommand.ConfigCommand("ftp-backup").execute(cliConn)
        clicommand.SimpleCommand("filesystem unmount {0}".format(srcDir)).execute(cliConn)
        cliConn.logout()
        del cliConn
    except:
        print("Error unmounting axigen backup filesystem:", sys.exc_info()[0])
    
    if not bkResult:
        sys.exit(3)
    sys.exit(0)
    
